#!/usr/local/bin/perl
#
# sendmail-larrd.pl - larrd module for collecting sendmail stats
#
# Scott Walters Copyright 1999
# scott@PacketPushers.com
#
# Christian Pearce
# pearcec@PacketPushers.com
#
#
# Do whatever you want with it, just don't sell it!
# I take no liability for it, use at your own risk!

# V1.0     Scott Walters & Christian Pearce
#
#  1.1 22 Jan 2003    Craig Cook
#      Major rewrite
#      - Uses sub procedures
#      - Added debug output
#
#  1.2 13 Mar 2003    Craig Cook
#      - changed $ENV{'DEBUG'} to $DEBUG
#
#  1.3 23 Jan 2004    Craig Cook
#      - do not read log into array - not efficient
#      - Applied patch by Richard Westlake
#          Change to the log file data scanning loop.
#          -Add code to update an rrd file for each mailer line,
#             rather then just the last mailer line.
#          -Add code to collect data from the totals line ("T"),
#             store this with a mailer name of Total
#             (rrd file name ends in .sendmail.Total.rrd)
#          -Add code to collect data from the connections line ("C"),
#             store this with a mailer name of Connections
#             (rrd file name ends in .sendmail.Connections.rrd)
#          -Add code to remove strange characters (e.g. *) from the
#             mailer name and skip some non-data lines,
#             which could confuse the collector.

# Set a path to the rrd tool lib
# Set as and $ENV and see what is up
use lib $ENV{'RRDPM'};
#use lib "/usr/local/rrdtool/lib/perl/";
use RRDs;

use lib $ENV{'LARRDPM'};
#use lib "/usr/local/larrd/lib/LARRD";
use LARRD;

my $larrd = new LARRD;

$ds = "sendmail";

$DEBUG = 1 if $ENV{'DEBUG'};

if ($DEBUG) {
  use Data::Dumper ;  # only for debug output
}

@ds = ("DS:msgsfr:DERIVE:600:0:U",
       "DS:bytes_from:DERIVE:600:0:U",
       "DS:msgsto:DERIVE:600:0:U",
       "DS:bytes_to:DERIVE:600:0:U",
       "DS:msgsrej:DERIVE:600:0:U",
       "DS:msgsdis:DERIVE:600:0:U");

my %h_fn;

##################################
# Start of script
##################################

$larrd->check_larrd_env;

##
# real work begins here
##

&read_bb_logs;
&read_data_logs;

print "$0: Data::Dumper->Dump([\%h_fn],[\"h_fn\"])" if $DEBUG ;

while ( ($host,$log) = each %h_fn ) {
    my $new_sm = 0;

    #  grab the timestamp from log file for rrd entry.
    &get_log_time;

    print "$0 : DEBUG host=$host, log=$log\n" if ($DEBUG);

    # Now lets do some work - process the list

    &process_log;

}

sub read_bb_logs {

  # go to the logs area for simplication of the list
  # Perl Cookbook Recipe 9.5
  opendir(DIR, "$ENV{'BBLOGS'}") or die "$0: $!";
  while ( defined($log = readdir(DIR))) {
        next unless $log =~ /\.$ds$/io;
        my $host = $log;
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        $h_fn{$host}="$ENV{'BBLOGS'}/$log";
  }
  closedir(DIR);

}

sub read_data_logs {

  opendir(DIR, "$ENV{'BBVAR'}/data") or die "$0: $!";
  while ( defined($log = readdir(DIR))) {
        next unless $log =~ /\.$ds$/io;
        my $host = $log;
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        $h_fn{$host}="$ENV{'BBVAR'}/data/$log";
  }

  closedir(DIR);

}


sub process_log {

  print "$0: DEBUG in process_log\n" if $DEBUG;

  print "$0 : DEBUG log=$log\n" if ($DEBUG);
  open(LOG,"$log");

  while (my $line = <LOG>) {

    chomp $line;

    last if ($line =~ /^(purple)/);
    # ensure colour is valid - ie don't graph purple results
    #if($line =~ /^green\s+|^yellow\s+|^red\s+/) {
    next if ($line =~ /^(green|red|yellow)/);  # Skip BB test colour line
    next if ($line =~ /^\s\s*$/);   # Skip blank lines

    print "$0: DEBUG line: $line\n" if $DEBUG;

    # looking for info like this:
    #
    #Statistics from Wed Jun 19 16:29:41 2002
    # M   msgsfr  bytes_from   msgsto    bytes_to  msgsrej msgsdis  Mailer
    # 3   183435     215701K        0          0K        0       0  local
    # 5        0          0K   183435     215544K        0       0  esmtp
    #=============================================================
    # T   183435     215701K   183435     215544K        0       0
    # C   183435               183435                    0

    ($M,$mfr,$bfr,$mt,$bt,$mr,$md,$mailer)=("","","","","","","","");
    chomp $line;

    if($line =~ /^Statistics/){ # skip this line
       next;
    }elsif($line =~ /Mailer$/){ # mailstat title line
#       Check which output format mailstats is using
       $new_sm = 1 if ($line =~ /msgsrej msgsdis/);
    }elsif($line =~ /^\s*[0-9]+\s+/){  #Mailer line
       if($new_sm){ ($M,$mfr,$bfr,$mt,$bt,$mr,$md,$mailer) = split " ", $line;
       }else{ ($M,$mfr,$bfr,$mt,$bt,$mailer) = split " ", $line;
       }
    }elsif($line =~ /^\=/){# Skip line between mailers and totals
       next;
    }elsif( $line =~ /^\sT\s/){ # totals line
       if($new_sm){($M,$mfr,$bfr,$mt,$bt,$mr,$md) = split " ", $line;
       }else{      ($M,$mfr,$bfr,$mt,$bt) = split " ", $line;
       }
       $mailer="Total";
    }elsif( $line =~ /^\sC\s/){ # Connections line
       if($new_sm){ ($M,$mfr,$mt,$mr) = split " ", $line;
       }else{       ($M,$mfr,$mt) = split " ", $line;
       }
       $mailer="Connections";  
    }elsif( $line =~ /^Status/){ #
#      We should never see a BB status line, but if we do then
#      there is no more data to process in this file
       last;
    }else{
#       I dont know what we are doing here so skip this log file line
       next;
    }

    print "$0: M: $M, mfr: $mfr, bfr: $bfr, mt: $mt, bt: $bt, mr: $mr, md: $md, mailer: $mailer" if ($DEBUG);
    if( $mailer ne "" ){
#       update the rrd filename
        &update_rrd;
    };
  }


  close(LOG);

  # delete data log file after we have used it
  unlink $log if $log =~ /$ENV{'BBVAR'}\/data/io;

}

sub get_log_time {
  # Sometimes you get it from stat, sometimes you get it from the log
  ($ctime) = (stat($log))[10];
  print "$0 : DEBUG ctime = $ctime\n" if ($DEBUG);
}

sub update_rrd {

# edit mailer string to remove strange characters
  $mailer =~ s/[\*\/\<\>]+/_/g;
  $mailer =~ s/__*/_/g;
#
  $RRD="$ENV{'BBRRDS'}/$host.$ds.$mailer.rrd";

  print "$0 : RRD=$RRD\n" if ($DEBUG);

  # if it doesn't exist, make it
  if ( not -f $RRD ) {
    &create_rrd;
  }

  my($rrdupdate)="$ctime:$mfr:$bfr:$mt:$bt:$mr:$md";

  print "$0 : DEBUG final update: $rrdupdate\n" if ($DEBUG);

  RRDs::update("$RRD","$rrdupdate");
  $ERR=RRDs::error;
  if($ERR) {
    print "$0: WARN updating $RRD: $ERR\n" if ($ENV{'WARN'});
  }

}

sub create_rrd {

  print "$0: DEBUG in create_rrd\n" if $DEBUG;

    # RRAS defined in larrd.pl
    @rras = split " ",$ENV{'RRAS'};

    RRDs::create($RRD,@ds,@rras);
    $ERR=RRDs::error;
    if($ERR) {
      print "$0: ERROR creating $RRD: $ERR\n" if ($ENV{'ERROR'});
      next;
    }

    $ctime="N";
    print "$0: STATUS did not find $RRD, created.\n" if ($ENV{'STATUS'});
}

##############################################
# end of script
##############################################
