#!/usr/local/bin/perl
#
# $Id: vmstat-larrd.pl,v 1.5 2004/02/24 20:30:31 cncook Exp $
#
# vmstat-larrd.pl: larrd module to collect disk utilization stats 
#
# Scott Walters Copyright 1999
# scott@PacketPushers.com 
#  
# Christian Pearce
# pearcec@PacketPushers.com
#
# Do whatever you want with it
# just don't sell it and make money off of it and not cut me in ;)
# I take no liability for it, use at your own risks
# 
# V1.1 Craig Cook
# Added $htovm{'suse'}=$htovm{'redhat'};
# Thanks to Tom Kauffman
#
# V1.2 Craig Cook
# 20 Jan 2004  Changed to use procedures
# 1.3  Craig Cook
# aix section: Changed from
#                        cpu_wait => 16,
#to:
#                        cpu_w    => 16,
# V1.4 Craig Cook
# 21 Apr 2005  Converted to use LARRD.pm

my $DEBUG = 0;
$DEBUG = 1 if $ENV{'DEBUG'};

if ($DEBUG) {
  $WARN = 1;
  use Data::Dumper;
  $ENV{'BBHOME'}="/usr/local/bb";
  $ENV{'BBVAR'}="/usr/local/bbvar";
  $ENV{'BBLOGS'}="/usr/local/bbvar/logs";
  $ENV{'BBRRDS'}="$ENV{'BBVAR'}/rrd";
  use lib "/usr/local/rrdtool/lib/perl/";
  use lib "/usr/local/larrd/lib/LARRD";
}

use lib $ENV{'RRDPM'};
use RRDs;

use lib $ENV{'LARRDPM'};
use LARRD;

my $larrd = new LARRD;

my $ds = "vmstat";

# Define some host to vmstat hash column translations

# Create a hash of hashes
%htovm = (solaris => { cpu_r     => 0,
                       cpu_b     => 1,
		       cpu_w     => 2,
		       mem_swap  => 3,
	   	       mem_free  => 4,
 		       mem_re    => 5,
		       mem_mf    => 6,
		       mem_pi    => 7,
		       mem_po    => 8,
		       sr        => 11,
                       cpu_int   => 16,
                       cpu_syc   => 17,
                       cpu_csw   => 18,
		       cpu_usr   => 19,
                       cpu_sys   => 20,
                       cpu_idl   => 21,
		     },
	  redhat => {  cpu_r    => 0,
                       cpu_b    => 1,
		       cpu_w    => 2,
		       mem_swpd => 3,
		       mem_free => 4,
           	       mem_buff => 5,
                       mem_cach => 6,
      		       mem_si   => 7,
                       mem_so   => 8,
		       dsk_bi   => 9,
		       dsk_bo   => 10,
                       cpu_int  => 11,
                       cpu_csw  => 12,
		       cpu_usr  => 13,
                       cpu_sys  => 14,
                       cpu_idl  => 15,
		     },
          osf => {     cpu_r    => 0, 
                       cpu_b    => 1, 
                       cpu_w    => 2, 
                       mem_free => 4, 
                       mem_mf   => 6, 
                       mem_pi   => 10, 
                       mem_po   => 11, 
                       cpu_int  => 12, 
                       cpu_syc  => 13, 
                       cpu_csw  => 14, 
                       cpu_usr  => 15, 
                       cpu_sys  => 16, 
                       cpu_idl  => 17, 
                 }, 
          aix     => {  cpu_r    => 0,
                        cpu_b    => 1,
                        mem_avm  => 2,
                        mem_free => 3,
                        mem_re   => 4,
                        mem_pi   => 5,
                        mem_po   => 6,
                        mem_fr   => 7,
                        sr       => 8,
                        mem_cy   => 9,
                        cpu_int  => 10,
                        cpu_syc  => 11,
                        cpu_csw  => 12,
                        cpu_usr  => 13,
                        cpu_sys  => 14,
                        cpu_idl  => 15,
                        cpu_w    => 16,
                     },
	  freebsd => {  cpu_r    => 0,
                        cpu_b    => 1,
                        cpu_w    => 2,
                        mem_avm  => 3,
                        mem_free => 4,
                        mem_flt  => 5,
                        mem_re   => 6,
                        mem_pi   => 7,
                        mem_po   => 8,
                        mem_fr   => 9,
                        sr       => 10,
                        dsk_da0  => 11,
                        dsk_fd0  => 12,
                        cpu_int  => 13,
                        cpu_csw  => 15,
                        cpu_sys  => 16,
                        cpu_usr  => 17,
                        cpu_idl  => 18,
                     },
	  hpux   => {  cpu_r    => 0,
                       cpu_b    => 1,
		       cpu_w    => 2,
		       mem_swpd => 3,
		       mem_free => 4,
           	       mem_buff => 5,
                       mem_cach => 6,
                       mem_pi   => 7,
                       mem_po   => 8,
                       mem_fr   => 9,
                       sr       => 11,
                       cpu_int  => 13,
                       cpu_csw  => 14,
		       cpu_usr  => 15,
                       cpu_sys  => 16,
                       cpu_idl  => 17,
		     },
	 );

# Aliases
$htovm{'debian'}=$htovm{'redhat'};
$htovm{'suse'}=$htovm{'redhat'};

my $reference_to_htovm = \%htovm;

##################################
# Start of script
##################################

$larrd->check_larrd_env;

##
# real work begins here
##

$filenames_ref = $larrd->get_bb_data_log_filenames($DEBUG);

$hash_ref_for_hosts = $larrd->read_bb_data_logs($DEBUG,$ds,$filenames_ref);

while ( ($host,$log_filename) = each %$hash_ref_for_hosts ) {

    $RRD="$ENV{'BBRRDS'}/$host.$ds.rrd";

    print "$0 : DEBUG host=$host, log=$log_filename\n" if ($DEBUG);

    $log_time = $larrd->get_log_time($DEBUG,$log_filename);

    $reference_to_log_data = $larrd->get_info_from_log($DEBUG,$log_filename);

    if ($log_filename =~ /\/data\//) {
      # Log data is in memory, we do not need the data file anymore
      unlink $log_filename;
    }

    # Reading from data dir, this is not needed
    #if($$reference_to_log_data[0] !~ /^(green|yellow|red)\s+/) {
            #print "$0: WARN $host is not showing red, yellow, or green. $log skipped.\n" if ($WARN);
            #next;
    #}

    # Now lets process the data

    my ($data_value,$reference_to_rrd_data_structure) = &process_log($DEBUG,$reference_to_log_data,$reference_to_htovm);

    if ($data_value ne "") {

       # If we are looking at a log with no larrdable data
       # do not create the rrd and in turn the graph
       # It makes sense this way.

       $larrd->update_rrd($DEBUG,$RRD,$log_time,$data_value,$reference_to_rrd_data_structure);

    } # end if ($data_value ne "") {


}

sub process_log ($$$) {
my ($DEBUG, $log_data_ref, $reference_to_htovm) = @_;
my $bb_host_type;
my @cols;
my @ds;
my $data_value_to_return = "";

    #Expect data like this:
    #solaris
    # 0 0 0  82360  9424   2 241 10 23 36  0 11  4 12  0  0  431 1263  340  4  8 88

    print "$0: DEBUG in process_log\n" if $DEBUG;

    $bb_host_type = $$reference_to_log_data[0];

    chomp $bb_host_type;

    print "$0: DEBUG bb_host_type = $bb_host_type\n" if $DEBUG;

    $line = $$reference_to_log_data[1];

    print "$0: DEBUG line: $line\n" if $DEBUG;

    chomp $line;

    @cols = split " ",$line;

    # since we do not know what type of OS data to expect until now, we have to build the @ds here

    foreach $col ( sort keys %{$reference_to_htovm->{ $bb_host_type }} ) {
      push @ds,"DS:$col:GAUGE:600:0:U";

      if ($data_value_to_return eq "") {
        $data_value_to_return = "$cols[$reference_to_htovm->{ $bb_host_type }{ $col }]";
      } else {
          $data_value_to_return = "$data_value_to_return:$cols[$reference_to_htovm->{ $bb_host_type }{ $col }]";
      }

    }

    $reference_to_rrd_data_structure = \@ds;

    print "$0: DEBUG data_value_to_return: $data_value_to_return\n" if $DEBUG;

    return ($data_value_to_return,$reference_to_rrd_data_structure);

}

##############################################
# end of script
##############################################
